import { isFullDatabaseObject, isFullUser } from './guards.js';
class Parser {
    constructor(map, schema) {
        this.map = map;
        this.schema = schema;
    }
    parseQueryDatabaseResponse(response) {
        return response.results.map((result) => {
            if (!isFullDatabaseObject(result)) {
                throw new Error('Encountered partial database object.');
            }
            const obj = this.parse(result.properties);
            return Object.assign(obj, { id: result.id });
        });
    }
    parse(record) {
        return Object.entries(this.schema).reduce((acc, [key, desc]) => {
            const value = record[desc.key];
            if (value.type !== desc.type) {
                throw Error('type does not match expected type');
            }
            const fn = this.map[desc.type];
            if (!fn) {
                throw new Error('No parser provided for type');
            }
            acc[key] = fn(value);
            return acc;
        }, {});
    }
}
function notImplementedFunction() {
    throw new Error('Not Implemented');
}
export class ParserBuilder {
    constructor(map) {
        this.map = map;
    }
    static create() {
        return new ParserBuilder({
            date: (v) => {
                return v.date ? new Date(v.date.start) : null;
            },
            number: (v) => { var _a; return (_a = v.number) !== null && _a !== void 0 ? _a : 0; },
            select: (v) => { var _a, _b; return (_b = (_a = v.select) === null || _a === void 0 ? void 0 : _a.name) !== null && _b !== void 0 ? _b : null; },
            multi_select: (v) => { var _a; return (_a = v.multi_select) === null || _a === void 0 ? void 0 : _a.map((v) => v.name); },
            title: (v) => (v.title.length > 0 ? v.title[0].plain_text : ''),
            people: (v) => v.people.map((person) => {
                var _a;
                if (isFullUser(person)) {
                    return (_a = person.name) !== null && _a !== void 0 ? _a : '';
                }
                return '';
            }),
            checkbox: (v) => v.checkbox,
            rich_text: (v) => v.rich_text.reduce((a, v) => (a += v.plain_text), ''),
            files: (v) => v.files.map((f) => {
                // Hacky hack, external URLs expire hourly
                // Need to download these
                return {
                    name: f.name,
                    url: f.type === 'external'
                        ? f.external.url
                        : f.type === 'file'
                            ? f.file.url
                            : '',
                };
            }),
            url: notImplementedFunction,
            formula: notImplementedFunction,
            relation: notImplementedFunction,
            rollup: notImplementedFunction,
            email: notImplementedFunction,
            phone_number: notImplementedFunction,
            last_edited_by: notImplementedFunction,
            last_edited_time: notImplementedFunction,
            created_by: notImplementedFunction,
            created_time: notImplementedFunction,
        });
    }
    addParserForType(type, fn) {
        this.map[type] = fn;
        return this;
    }
    build(schema) {
        return new Parser(this.map, schema);
    }
}
//# sourceMappingURL=parser.js.map